@@ -32,7 +32,15 @@ class AccountController extends Controller
                 'title' => $this->translate('List and configure your own navigation items'),
                 'label' => $this->translate('Navigation'),
                 'url'   => 'navigation'
-            ));
+            ))
+            ->add(
+                'devices',
+                array(
+                    'title' => $this->translate('List of devices you are logged in'),
+                    'label' => $this->translate('My Devices'),
+                    'url'   => 'my-devices'
+                )
+            );
     }
 
     /**
@@ -43,7 +51,7 @@ class AccountController extends Controller
         $config = Config::app()->getSection('global');
         $user = $this->Auth()->getUser();
         if ($user->getAdditional('backend_type') === 'db') {
-            if ($user->can('*') || ! $user->can('no-user/password-change')) {
+            if ($user->can('user/password-change')) {
                 try {
                     $userBackend = UserBackend::create($user->getAdditional('backend_name'));
                 } catch (ConfigurationError $e) {
@@ -61,9 +69,9 @@ class AccountController extends Controller
 
         $form = new PreferenceForm();
         $form->setPreferences($user->getPreferences());
-        if ($config->get('config_backend', 'ini') !== 'none') {
+        if ($config->get('config_backend', 'db') !== 'none') {
             $form->setStore(PreferencesStore::create(new ConfigObject(array(
-                'store'     => $config->get('config_backend', 'ini'),
+                'store'     => $config->get('config_backend', 'db'),
                 'resource'  => $config->config_resource
             )), $user));
         }
