@@ -4,6 +4,7 @@
 namespace Icinga\Forms\Config;
 
 use Icinga\Forms\Config\General\ApplicationConfigForm;
+use Icinga\Forms\Config\General\DefaultAuthenticationDomainConfigForm;
 use Icinga\Forms\Config\General\LoggingConfigForm;
 use Icinga\Forms\Config\General\ThemingConfigForm;
 use Icinga\Forms\ConfigForm;
@@ -30,8 +31,19 @@ class GeneralConfigForm extends ConfigForm
         $appConfigForm = new ApplicationConfigForm();
         $loggingConfigForm = new LoggingConfigForm();
         $themingConfigForm = new ThemingConfigForm();
+        $domainConfigForm = new DefaultAuthenticationDomainConfigForm();
         $this->addSubForm($appConfigForm->create($formData));
         $this->addSubForm($loggingConfigForm->create($formData));
         $this->addSubForm($themingConfigForm->create($formData));
+        $this->addSubForm($domainConfigForm->create($formData));
+    }
+
+    public function onRequest()
+    {
+        parent::onRequest();
+
+        if ($this->config->getConfigObject()->global->config_backend === 'ini') {
+            $this->warning('The preferences backend of type INI is deprecated and will be removed with version 2.11');
+        }
     }
 }
