@@ -4,24 +4,25 @@
 namespace Icinga\Application\Modules;
 
 use Exception;
-use Zend_Controller_Router_Route;
-use Zend_Controller_Router_Route_Abstract;
-use Zend_Controller_Router_Route_Regex;
 use Icinga\Application\ApplicationBootstrap;
 use Icinga\Application\Config;
+use Icinga\Application\Hook;
 use Icinga\Application\Icinga;
 use Icinga\Application\Logger;
-use Icinga\Application\Modules\DashboardContainer;
-use Icinga\Application\Modules\MenuItemContainer;
 use Icinga\Exception\IcingaException;
 use Icinga\Exception\ProgrammingError;
 use Icinga\Module\Setup\SetupWizard;
 use Icinga\Util\File;
-use Icinga\Util\Translator;
-use Icinga\Web\Controller\Dispatcher;
-use Icinga\Application\Hook;
 use Icinga\Web\Navigation\Navigation;
 use Icinga\Web\Widget;
+use ipl\I18n\GettextTranslator;
+use ipl\I18n\StaticTranslator;
+use ipl\I18n\Translation;
+use RecursiveDirectoryIterator;
+use RecursiveIteratorIterator;
+use Zend_Controller_Router_Route;
+use Zend_Controller_Router_Route_Abstract;
+use Zend_Controller_Router_Route_Regex;
 
 /**
  * Module handling
@@ -30,6 +31,11 @@ use Icinga\Web\Widget;
  */
 class Module
 {
+    use Translation {
+        translate as protected;
+        translatePlural as protected;
+    }
+
     /**
      * Module name
      *
@@ -44,6 +50,13 @@ class Module
      */
     private $basedir;
 
+    /**
+     * Directory for assets
+     *
+     * @var string
+     */
+    private $assetDir;
+
     /**
      * Directory for styles
      *
@@ -51,6 +64,13 @@ class Module
      */
     private $cssdir;
 
+    /**
+     * Directory for Javascript
+     *
+     * @var string
+     */
+    private $jsdir;
+
     /**
      * Base application directory
      *
@@ -184,6 +204,34 @@ class Module
      */
     protected $jsFiles = array();
 
+    /**
+     * Globally provided CSS assets
+     *
+     * @var array
+     */
+    protected $cssAssets = [];
+
+    /**
+     * Globally provided JS assets
+     *
+     * @var array
+     */
+    protected $jsAssets = [];
+
+    /**
+     * Required CSS assets
+     *
+     * @var array
+     */
+    protected $cssRequires = [];
+
+    /**
+     * Required JS assets
+     *
+     * @var array
+     */
+    protected $jsRequires = [];
+
     /**
      * Routes to add to the route chain
      *
@@ -247,6 +295,7 @@ class Module
         $this->app            = $app;
         $this->name           = $name;
         $this->basedir        = $basedir;
+        $this->assetDir       = $basedir . '/asset';
         $this->cssdir         = $basedir . '/public/css';
         $this->jsdir          = $basedir . '/public/js';
         $this->libdir         = $basedir . '/library';
@@ -258,6 +307,8 @@ class Module
         $this->runScript      = $basedir . '/run.php';
         $this->configScript   = $basedir . '/configuration.php';
         $this->metadataFile   = $basedir . '/module.info';
+
+        $this->translationDomain = $name;
     }
 
     /**
@@ -436,6 +487,7 @@ class Module
             return false;
         }
         $this->registerWebIntegration();
+        $this->registerAssets();
         $this->registered = true;
 
         return true;
@@ -469,7 +521,7 @@ class Module
      * @param string $name
      * @param bool   $autoload
      *
-     * @return mixed
+     * @return  self
      *
      * @throws ProgrammingError When the module is not yet loaded
      */
@@ -485,6 +537,164 @@ class Module
         return $manager->getModule($name);
     }
 
+    /**
+     * Provide a static CSS asset which can be required by other modules
+     *
+     * @param   string  $path   The path, relative to the module's base
+     *
+     * @return  $this
+     */
+    protected function provideCssAsset($path)
+    {
+        $fullPath = join(DIRECTORY_SEPARATOR, [$this->basedir, $path]);
+        $this->cssAssets[] = $fullPath;
+        $this->cssRequires[] = $fullPath; // A module should not have to require its own assets
+
+        return $this;
+    }
+
+    /**
+     * Get the CSS assets provided by this module
+     *
+     * @return array
+     */
+    public function getCssAssets()
+    {
+        return $this->cssAssets;
+    }
+
+    /**
+     * Require CSS from a different module
+     *
+     * @param   string  $path   The file's path, relative to the module's asset or base directory
+     * @param   string  $from   The module's name
+     *
+     * @deprecated Deprecated with v2.9, don't use and depend on a library instead
+     * @return  $this
+     */
+    protected function requireCssFile($path, $from)
+    {
+        $module = self::get($from);
+        $cssAssetDir = join(DIRECTORY_SEPARATOR, [$module->assetDir, 'css']);
+        foreach ($module->getCssAssets() as $assetPath) {
+            if (substr($assetPath, 0, strlen($cssAssetDir)) === $cssAssetDir) {
+                $relativePath = ltrim(substr($assetPath, strlen($cssAssetDir)), '/\\');
+            } else {
+                $relativePath = ltrim(substr($assetPath, strlen($module->basedir)), '/\\');
+            }
+
+            if ($path === $relativePath) {
+                $this->cssRequires[] = $assetPath;
+                break; // Exact match, won't match again..
+            } elseif (fnmatch($path, $relativePath)) {
+                $this->cssRequires[] = $assetPath;
+            }
+        }
+
+        return $this;
+    }
+
+    /**
+     * Check whether this module requires CSS from a different module
+     *
+     * @return bool
+     */
+    public function requiresCss()
+    {
+        $this->launchConfigScript();
+        return ! empty($this->cssRequires);
+    }
+
+    /**
+     * List the CSS assets required by this module
+     *
+     * @return array
+     */
+    public function getCssRequires()
+    {
+        $this->launchConfigScript();
+        return $this->cssRequires;
+    }
+
+    /**
+     * Provide a static Javascript asset which can be required by other modules
+     *
+     * @param   string  $path   The path, relative to the module's base
+     *
+     * @return  $this
+     */
+    protected function provideJsAsset($path)
+    {
+        $fullPath = join(DIRECTORY_SEPARATOR, [$this->basedir, $path]);
+        $this->jsAssets[] = $fullPath;
+        $this->jsRequires[] = $fullPath; // A module should not have to require its own assets
+
+        return $this;
+    }
+
+    /**
+     * Get the Javascript assets provided by this module
+     *
+     * @return array
+     */
+    public function getJsAssets()
+    {
+        return $this->jsAssets;
+    }
+
+    /**
+     * Require Javascript from a different module
+     *
+     * @param   string  $path   The file's path, relative to the module's asset or base directory
+     * @param   string  $from   The module's name
+     *
+     * @deprecated Deprecated with v2.9, don't use and depend on a library instead
+     * @return  $this
+     */
+    protected function requireJsFile($path, $from)
+    {
+        $module = self::get($from);
+        $jsAssetDir = join(DIRECTORY_SEPARATOR, [$module->assetDir, 'js']);
+        foreach ($module->getJsAssets() as $assetPath) {
+            if (substr($assetPath, 0, strlen($jsAssetDir)) === $jsAssetDir) {
+                $relativePath = ltrim(substr($assetPath, strlen($jsAssetDir)), '/\\');
+            } else {
+                $relativePath = ltrim(substr($assetPath, strlen($module->basedir)), '/\\');
+            }
+
+            if ($path === $relativePath) {
+                $this->jsRequires[] = $assetPath;
+                break; // Exact match, won't match again..
+            } elseif (fnmatch($path, $relativePath)) {
+                $this->jsRequires[] = $assetPath;
+            }
+        }
+
+        return $this;
+    }
+
+    /**
+     * Check whether this module requires Javascript from a different module
+     *
+     * @return bool
+     */
+    public function requiresJs()
+    {
+        $this->launchConfigScript();
+        return ! empty($this->jsRequires);
+    }
+
+    /**
+     * List the Javascript assets required by this module
+     *
+     * @return array
+     */
+    public function getJsRequires()
+    {
+        $this->launchConfigScript();
+        return $this->jsRequires;
+    }
+
     /**
      * Provide an additional CSS/LESS file
      *
@@ -643,12 +853,50 @@ class Module
      * Get the module dependencies
      *
      * @return array
+     * @deprecated Use method getRequiredModules() instead
      */
     public function getDependencies()
     {
         return $this->metadata()->depends;
     }
 
+    /**
+     * Get required libraries
+     *
+     * @return array
+     */
+    public function getRequiredLibraries()
+    {
+        $requiredLibraries = $this->metadata()->libraries;
+
+        // Register module requirements for ipl and reactbundle as library requirements
+        $requiredModules = $this->metadata()->modules ?: $this->metadata()->depends;
+        if (isset($requiredModules['ipl']) && ! isset($requiredLibraries['icinga-php-library'])) {
+            $requiredLibraries['icinga-php-library'] = $requiredModules['ipl'];
+        }
+
+        if (isset($requiredModules['reactbundle']) && ! isset($requiredLibraries['icinga-php-thirdparty'])) {
+            $requiredLibraries['icinga-php-thirdparty'] = $requiredModules['reactbundle'];
+        }
+
+        return $requiredLibraries;
+    }
+
+    /**
+     * Get required modules
+     *
+     * @return array
+     */
+    public function getRequiredModules()
+    {
+        $requiredModules = $this->metadata()->modules ?: $this->metadata()->depends;
+
+        // Both modules are deprecated and their successors are now dependencies of web itself
+        unset($requiredModules['ipl'], $requiredModules['reactbundle']);
+
+        return $requiredModules;
+    }
+
     /**
      * Fetch module metadata
      *
@@ -657,18 +905,21 @@ class Module
     protected function metadata()
     {
         if ($this->metadata === null) {
-            $metadata = (object) array(
-                'name'        => $this->getName(),
-                'version'     => '0.0.0',
-                'title'       => null,
-                'description' => '',
-                'depends'     => array(),
-            );
+            $metadata = (object) [
+                'name'          => $this->getName(),
+                'version'       => '0.0.0',
+                'title'         => null,
+                'description'   => '',
+                'depends'       => [],
+                'libraries'     => [],
+                'modules'       => []
+            ];
 
             if (file_exists($this->metadataFile)) {
                 $key = null;
+                $simpleRequires = false;
                 $file = new File($this->metadataFile, 'r');
-                foreach ($file as $line) {
+                foreach ($file as $lineno => $line) {
                     $line = rtrim($line);
 
                     if ($key === 'description') {
@@ -683,27 +934,65 @@ class Module
                         continue;
                     }
 
-                    list($key, $val) = preg_split('/:\s+/', $line, 2);
-                    $key = lcfirst($key);
+                    if (strpos($line, ':') === false) {
+                        Logger::debug(
+                            "Can't process line %d in %s: Line does not specify a key:value pair"
+                            . " nor is it part of the description (indented with a single space)",
+                            $lineno,
+                            $this->metadataFile
+                        );
 
+                        break;
+                    }
+
+                    $parts = preg_split('/:\s+/', $line, 2);
+                    if (count($parts) === 1) {
+                        $parts[] = '';
+                    }
+
+                    list($key, $val) = $parts;
+
+                    $key = strtolower($key);
                     switch ($key) {
+                        case 'requires':
+                            if ($val) {
+                                $simpleRequires = true;
+                                $key = 'libraries';
+                            } else {
+                                break;
+                            }
+
+                            // Shares the syntax with `Depends`
+                        case '  libraries':
+                        case '  modules':
+                            if ($simpleRequires && $key[0] === ' ') {
+                                Logger::debug(
+                                    'Can\'t process line %d in %s: Requirements already registered by a previous line',
+                                    $lineno,
+                                    $this->metadataFile
+                                );
+                                break;
+                            }
+
+                            $key = ltrim($key);
+                            // Shares the syntax with `Depends`
                         case 'depends':
                             if (strpos($val, ' ') === false) {
-                                $metadata->depends[$val] = true;
+                                $metadata->{$key}[$val] = true;
                                 continue 2;
                             }
 
                             $parts = preg_split('/,\s+/', $val);
                             foreach ($parts as $part) {
-                                if (preg_match('/^(\w+)\s+\((.+)\)$/', $part, $m)) {
-                                    $metadata->depends[$m[1]] = $m[2];
+                                if (preg_match('/^([\w\-\/]+)\s+\((.+)\)$/', $part, $m)) {
+                                    $metadata->{$key}[$m[1]] = $m[2];
                                 } else {
                                     // TODO: FAIL?
                                     continue;
                                 }
                             }
-                            break;
 
+                            break;
                         case 'description':
                             if ($metadata->title === null) {
                                 $metadata->title = $val;
@@ -723,8 +1012,6 @@ class Module
             }
 
             if ($metadata->description === '') {
-                // TODO: Check whether the translation module is able to
-                //       extract this
                 $metadata->description = t(
                     'This module has no description'
                 );
@@ -745,6 +1032,26 @@ class Module
         return $this->cssdir;
     }
 
+    /**
+     * Get the module's JS directory
+     *
+     * @return string
+     */
+    public function getJsDir()
+    {
+        return $this->jsdir;
+    }
+
+    /**
+     * Get the module's JS asset directory
+     *
+     * @return string
+     */
+    public function getJsAssetDir()
+    {
+        return join(DIRECTORY_SEPARATOR, [$this->assetDir, 'js']);
+    }
+
     /**
      * Get the module's controller directory
      *
@@ -1096,6 +1403,40 @@ class Module
         return $this;
     }
 
+    /**
+     * Register this module's assets
+     *
+     * @return $this
+     */
+    protected function registerAssets()
+    {
+        if (! is_dir($this->assetDir)) {
+            return $this;
+        }
+
+        $listAssets = function ($type) {
+            $dir = join(DIRECTORY_SEPARATOR, [$this->assetDir, $type]);
+            if (! is_dir($dir)) {
+                return [];
+            }
+
+            return new RecursiveIteratorIterator(new RecursiveDirectoryIterator(
+                $dir,
+                RecursiveDirectoryIterator::CURRENT_AS_PATHNAME | RecursiveDirectoryIterator::SKIP_DOTS
+            ));
+        };
+
+        foreach ($listAssets('css') as $assetPath) {
+            $this->provideCssAsset(ltrim(substr($assetPath, strlen($this->basedir)), '/\\'));
+        }
+
+        foreach ($listAssets('js') as $assetPath) {
+            $this->provideJsAsset(ltrim(substr($assetPath, strlen($this->basedir)), '/\\'));
+        }
+
+        return $this;
+    }
+
     /**
      * Bind text domain for i18n
      *
@@ -1103,9 +1444,10 @@ class Module
      */
     protected function registerLocales()
     {
-        if ($this->hasLocales()) {
-            Translator::registerDomain($this->name, $this->localedir);
+        if ($this->hasLocales() && StaticTranslator::$instance instanceof GettextTranslator) {
+            StaticTranslator::$instance->addTranslationDirectory($this->localedir, $this->name);
         }
+
         return $this;
     }
 
@@ -1318,22 +1660,4 @@ class Module
         $this->routes[$name] = $route;
         return $this;
     }
-
-    /**
-     * (non-PHPDoc)
-     * @see Translator::translate() For the function documentation.
-     */
-    protected function translate($string, $context = null)
-    {
-        return mt($this->name, $string, $context);
-    }
-
-    /**
-     * (non-PHPDoc)
-     * @see Translator::translatePlural() For the function documentation.
-     */
-    protected function translatePlural($textSingular, $textPlural, $number, $context = null)
-    {
-        return mtp($this->name, $textSingular, $textPlural, $number, $context);
-    }
 }
