@@ -3,10 +3,9 @@
 
 namespace Icinga\Web;
 
+use Icinga\Application\Icinga;
 use Icinga\Application\Logger;
-use RecursiveArrayIterator;
-use RecursiveIteratorIterator;
-use lessc;
+use Icinga\Util\LessParser;
 
 /**
  * Compile LESS into CSS
@@ -18,7 +17,7 @@ class LessCompiler
     /**
      * lessphp compiler
      *
-     * @var lessc
+     * @var LessParser
      */
     protected $lessc;
 
@@ -50,6 +49,8 @@ class LessCompiler
      */
     protected $source;
 
+    private $defaultThemeOverride = false;
+
     /**
      * Path of the LESS theme
      *
@@ -57,13 +58,19 @@ class LessCompiler
      */
     protected $theme;
 
+    /**
+     * Path of the LESS theme mode
+     *
+     * @var string
+     */
+    protected $themeMode;
+
     /**
      * Create a new LESS compiler
      */
     public function __construct()
     {
-        require_once 'lessphp/lessc.inc.php';
-        $this->lessc = new lessc();
+        $this->lessc = new LessParser();
         // Discourage usage of import because we're caching based on an explicit list of LESS files to compile
         $this->lessc->importDisabled = true;
     }
@@ -130,9 +137,22 @@ class LessCompiler
         if ($this->theme !== null) {
             $lessFiles[] = $this->theme;
         }
+
+        if ($this->themeMode !== null) {
+            $lessFiles[] = $this->themeMode;
+        }
+
         return $lessFiles;
     }
 
+    /**
+     * @deprecated Don't use
+     */
+    public function enableDefaultThemeOverride()
+    {
+        $this->defaultThemeOverride = true;
+    }
+
     /**
      * Set the path to the LESS theme
      *
@@ -150,6 +170,23 @@ class LessCompiler
         return $this;
     }
 
+    /**
+     * Set the path to the LESS theme mode
+     *
+     * @param   string  $themeMode  Path to the LESS theme mode
+     *
+     * @return  $this
+     */
+    public function setThemeMode($themeMode)
+    {
+        if (is_file($themeMode) && is_readable($themeMode)) {
+            $this->themeMode = $themeMode;
+        } else {
+            Logger::error('Can\t load theme mode %s. Make sure that the theme mode exists and is readable', $themeMode);
+        }
+        return $this;
+    }
+
     /**
      * Instruct the compiler to minify CSS
      *
@@ -173,10 +210,11 @@ class LessCompiler
         }
 
         $moduleCss = '';
+        $exportedVars = [];
         foreach ($this->moduleLessFiles as $moduleName => $moduleLessFiles) {
             $moduleCss .= '.icinga-module.module-' . $moduleName . ' {';
 
-            // TODO: Import these. (https://github.com/leafo/lessphp/issues/515)
+            // TODO: Import these.
             foreach ($this->moduleRequires as $requiredFile => $modules) {
                 if (in_array($moduleName, $modules, true)) {
                     $moduleCss .= file_get_contents($requiredFile);
@@ -184,17 +222,50 @@ class LessCompiler
             }
 
             foreach ($moduleLessFiles as $moduleLessFile) {
-                $moduleCss .= file_get_contents($moduleLessFile);
+                $content = file_get_contents($moduleLessFile);
+
+                $pattern = '/^@exports:\s*{((?:\s*@[^:}]+:[^;]*;\s+)+)};$/m';
+                if (preg_match_all($pattern, $content, $matches, PREG_SET_ORDER)) {
+                    foreach ($matches as $match) {
+                        $content = str_replace($match[0], '', $content);
+                        foreach (explode("\n", trim($match[1])) as $line) {
+                            list($name, $value) = explode(':', $line, 2);
+                            $exportedVars[trim($name)] = trim($value, ' ;');
+                        }
+                    }
+                }
+
+                $moduleCss .= $content;
             }
+
             $moduleCss .= '}';
         }
 
         $this->source .= $moduleCss;
 
+        $varExports = '';
+        foreach ($exportedVars as $name => $value) {
+            $varExports .= sprintf("%s: %s;\n", $name, $value);
+        }
+
+        // exported vars are injected at the beginning to avoid that they are
+        // able to override other variables, that's what themes are for
+        $this->source = $varExports . "\n\n" . $this->source;
+
+        if ($this->defaultThemeOverride) {
+            $this->source .= file_get_contents(
+                Icinga::app()->getBaseDir('public/css/icinga') . '/legacy-theme.less'
+            );
+        }
+
         if ($this->theme !== null) {
             $this->source .= file_get_contents($this->theme);
         }
 
+        if ($this->themeMode !== null) {
+            $this->source .= file_get_contents($this->themeMode);
+        }
+
         return preg_replace(
             '/(\.icinga-module\.module-[^\s]+) (#layout\.[^\s]+)/m',
             '\2 \1',
