@@ -21,15 +21,48 @@ class StyleSheet
      */
     const DEFAULT_THEME = 'Icinga';
 
+    /**
+     * The name of the default theme mode
+     *
+     * @var string
+     */
+    const DEFAULT_MODE = 'dark';
+
+    /**
+     * The themes that are compatible with the default theme
+     *
+     * @var array
+     */
+    const THEME_WHITELIST = [
+        'colorblind',
+        'high-contrast',
+        'solarized-dark',
+        'Winter'
+    ];
+
+    /**
+     * RegEx pattern for matching full css @media query of theme mode
+     *
+     * @var string
+     */
+    const REGEX_ALL_MODE_QUERY = '/@media\s*\(\s*min-height\s*:\s*@prefer-light-color-scheme\s*\)\s*,' .
+    '\s*\(\s*prefers-color-scheme\s*:\s*light\s*\)\s*and\s*\(\s*min-height\s*:\s*@enable-color-preference\s*\)/';
+
+    /**
+     * RegEx pattern for matching css @media query of theme mode
+     *
+     * @var string
+     */
+    const REGEX_AUTO_MODE_QUERY = '/@media.*prefers-color-scheme/';
+
     /**
      * Array of core LESS files Web 2 sends to the client
      *
      * @var string[]
      */
-    protected static $lessFiles = array(
+    protected static $lessFiles = [
         '../application/fonts/fontello-ifont/css/ifont-embedded.css',
         'css/vendor/normalize.css',
-        'css/vendor/tipsy.css',
         'css/icinga/base.less',
         'css/icinga/badges.less',
         'css/icinga/mixins.less',
@@ -51,8 +84,11 @@ class StyleSheet
         'css/icinga/spinner.less',
         'css/icinga/compat.less',
         'css/icinga/print.less',
-        'css/icinga/responsive.less'
-    );
+        'css/icinga/responsive.less',
+        'css/icinga/modal.less',
+        'css/icinga/audit.less',
+        'css/icinga/health.less'
+    ];
 
     /**
      * Application instance
@@ -80,14 +116,11 @@ class StyleSheet
      */
     public function __construct()
     {
-        // PHP had a rather conservative PCRE backtrack limit unless 5.3.7
-        if (version_compare(PHP_VERSION, '5.3.7') <= 0) {
-            ini_set('pcre.backtrack_limit', 1000000);
-        }
         $app = Icinga::app();
         $this->app = $app;
         $this->lessCompiler = new LessCompiler();
-        $this->pubPath = $app->getBootstrapDirectory();
+        $this->lessCompiler->enableDefaultThemeOverride();
+        $this->pubPath = $app->getBaseDir('public');
         $this->collect();
     }
 
@@ -96,6 +129,12 @@ class StyleSheet
      */
     protected function collect()
     {
+        foreach ($this->app->getLibraries() as $library) {
+            foreach ($library->getCssAssets() as $lessFile) {
+                $this->lessCompiler->addLessFile($lessFile);
+            }
+        }
+
         foreach (self::$lessFiles as $lessFile) {
             $this->lessCompiler->addLessFile($this->pubPath . '/' . $lessFile);
         }
@@ -108,6 +147,12 @@ class StyleSheet
                     $this->lessCompiler->addModuleLessFile($moduleName, $lessFilePath);
                 }
             }
+
+            if ($module->requiresCss()) {
+                foreach ($module->getCssRequires() as $lessFilePath) {
+                    $this->lessCompiler->addModuleRequire($moduleName, $lessFilePath);
+                }
+            }
         }
 
         $themingConfig = $this->app->getConfig()->getSection('themes');
@@ -127,18 +172,30 @@ class StyleSheet
             }
         }
 
-        if ($theme && $theme !== self::DEFAULT_THEME) {
-            if (($pos = strpos($theme, '/')) !== false) {
-                $moduleName = substr($theme, 0, $pos);
-                $theme = substr($theme, $pos + 1);
-                if ($mm->hasLoaded($moduleName)) {
-                    $module = $mm->getModule($moduleName);
-                    $this->lessCompiler->setTheme($module->getCssDir() . '/themes/' . $theme . '.less');
+        if ($themePath = self::getThemeFile($theme)) {
+            $this->lessCompiler->setTheme($themePath);
+        }
+
+        if (! $themePath || in_array($theme, self::THEME_WHITELIST, true)) {
+            $this->lessCompiler->addLessFile('css/icinga/login-orbs.less');
+        }
+
+        $mode = 'none';
+        if ($user = Auth::getInstance()->getUser()) {
+            $file = $themePath !== null ? file_get_contents($themePath) : '';
+            if ($file !== '' && ! preg_match(self::REGEX_ALL_MODE_QUERY, $file)) {
+                if (preg_match(self::REGEX_AUTO_MODE_QUERY, $file)) {
+                    $mode = 'system';
                 }
             } else {
-                $this->lessCompiler->setTheme($this->pubPath . '/css/themes/' . $theme . '.less');
+                $mode = $user->getPreferences()->getValue('icingaweb', 'theme_mode', self::DEFAULT_MODE);
             }
         }
+
+        // Temporarily disabled. Re-enabled with v2.10
+        $mode = 'none';
+
+        $this->lessCompiler->setThemeMode($this->pubPath . '/css/modes/'. $mode . '.less');
     }
 
     /**
@@ -182,6 +239,7 @@ class StyleSheet
 
         $request = $styleSheet->app->getRequest();
         $response = $styleSheet->app->getResponse();
+        $response->setHeader('Cache-Control', 'private,no-cache,must-revalidate', true);
 
         $noCache = $request->getHeader('Cache-Control') === 'no-cache' || $request->getHeader('Pragma') === 'no-cache';
 
@@ -194,9 +252,7 @@ class StyleSheet
 
         $etag = FileCache::etagForFiles($styleSheet->lessCompiler->getLessFiles());
 
-        $response
-            ->setHeader('Cache-Control', 'public', true)
-            ->setHeader('ETag', $etag, true)
+        $response->setHeader('ETag', $etag, true)
             ->setHeader('Content-Type', 'text/css', true);
 
         $cacheFile = 'icinga-' . $etag . ($minified ? '.min' : '') . '.css';
@@ -227,4 +283,32 @@ class StyleSheet
             return IcingaException::describe($e);
         }
     }
+
+    /**
+     * Get the path to the current LESS theme file
+     *
+     * @param $theme
+     *
+     * @return  string|null Return null if self::DEFAULT_THEME is set as theme, path otherwise
+     */
+    public static function getThemeFile($theme)
+    {
+        $app = Icinga::app();
+
+        if ($theme && $theme !== self::DEFAULT_THEME) {
+            if (($pos = strpos($theme, '/')) !== false) {
+                $moduleName = substr($theme, 0, $pos);
+                $theme = substr($theme, $pos + 1);
+                if ($app->getModuleManager()->hasLoaded($moduleName)) {
+                    $module = $app->getModuleManager()->getModule($moduleName);
+
+                    return $module->getCssDir() . '/themes/' . $theme . '.less';
+                }
+            } else {
+                return $app->getBaseDir('public') . '/css/themes/' . $theme . '.less';
+            }
+        }
+
+        return null;
+    }
 }
