@@ -2,29 +2,23 @@
 /* Icinga Web 2 | (c) 2013 Icinga Development Team | GPLv2+ */
 
 use Icinga\Web\Dom\DomNodeIterator;
-use Icinga\Module\Monitoring\Web\Helper\PluginOutputPurifier;
+use Icinga\Web\View;
+use Icinga\Web\Helper\HtmlPurifier;
 
 /**
  * Plugin output renderer
  */
 class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
 {
-    /**
-     * The return value of getPurifier()
-     *
-     * @var HTMLPurifier
-     */
-    protected static $purifier;
-
     /**
      * Patterns to be replaced in plain text plugin output
      *
      * @var array
      */
     protected static $txtPatterns = array(
-        '~\\\n~',
+        '~\\\n\\\n~',
         '~\\\t~',
-         '~\\\n\\\n~',
+        '~\\\n~',
         '~(\[|\()OK(\]|\))~',
         '~(\[|\()WARNING(\]|\))~',
         '~(\[|\()CRITICAL(\]|\))~',
@@ -58,9 +52,9 @@ class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
      * @var array
      */
     protected static $htmlPatterns = array(
-        '~\\\n~',
-        '~\\\t~',
         '~\\\n\\\n~',
+        '~\\\t~',
+        '~\\\n~',
         '~<table~'
     );
 
@@ -101,13 +95,12 @@ class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
         if ($command !== null) {
             $output = $this->hookRenderer->render($command, $output, ! $raw);
         }
-        $output = preg_replace('~<br[^>]*>~', "\n", $output);
-        if (preg_match('~<[^>]*["/\'][^>]*>~', $output)) {
+        if (preg_match('~<\w+(?>\s\w+=[^>]*)?>~', $output)) {
             // HTML
             $output = preg_replace(
                 self::$htmlPatterns,
                 self::$htmlReplacements,
-                PluginOutputPurifier::process($output)
+                HtmlPurifier::process($output)
             );
             $isHtml = true;
         } else {
@@ -115,7 +108,8 @@ class Zend_View_Helper_PluginOutput extends Zend_View_Helper_Abstract
             $output = preg_replace(
                 self::$txtPatterns,
                 self::$txtReplacements,
-                $this->view->escape($output)
+                // Not using the view here to escape this. The view sets `double_encode` to true
+                htmlspecialchars($output, ENT_COMPAT | ENT_SUBSTITUTE | ENT_HTML5, View::CHARSET, false)
             );
             $isHtml = false;
         }
