@@ -14,7 +14,6 @@
 
         this.searchValue = '';
         this.searchTimer = null;
-        this.initializeModules = true;
     };
 
     Icinga.Events.prototype = {
@@ -24,99 +23,34 @@
          */
         initialize: function () {
             this.applyGlobalDefaults();
-            $('#layout').trigger('rendered');
-            //$('.container').trigger('rendered');
-            $('.container').each(function(idx, el) {
-                icinga.ui.initializeControls($(el));
-            });
-        },
-
-        // TODO: What's this?
-        applyHandlers: function (event) {
-            var $target = $(event.target);
-            var _this = event.data.self;
-            var icinga = _this.icinga;
-
-            if (! icinga) {
-                // Attempt to catch a rare error, race condition, whatever
-                console.log('Got no icinga in applyHandlers');
-                return;
-            }
-
-            if (_this.initializeModules) {
-                var loaded = false;
-                var moduleName = $target.data('icingaModule');
-                if (moduleName) {
-                    if (icinga.hasModule(moduleName) && !icinga.isLoadedModule(moduleName)) {
-                        loaded |= icinga.loadModule(moduleName);
-                    }
-                }
-
-                $('.icinga-module', $target).each(function(idx, mod) {
-                    moduleName = $(mod).data('icingaModule');
-                    if (icinga.hasModule(moduleName) && !icinga.isLoadedModule(moduleName)) {
-                        loaded |= icinga.loadModule(moduleName);
-                    }
-                });
-
-                if (loaded) {
-                    // Modules may register their own handler for the 'renderend' event
-                    // so we need to ensure that it is called the first time they are
-                    // initialized
-                    event.stopImmediatePropagation();
-                    _this.initializeModules = false;
-
-                    var $container = $target.closest('.container');
-                    if (! $container.length) {
-                        // The page obviously got loaded for the first time,
-                        // so we'll trigger the event for all containers
-                        $container = $('.container');
-                    }
-
-                    $container.trigger('rendered');
-
-                    // But since we're listening on this event by ourself, we'll have
-                    // to abort our own processing as we'll process it twice otherwise
-                    return false;
-                }
-            } else {
-                _this.initializeModules = true;
-            }
-
-            $('.dashboard > div', $target).each(function(idx, el) {
-                var $element = $(el);
-                var $url = $element.data('icingaUrl');
-                if (typeof $url !== 'undefined') {
-                    icinga.loader.loadUrl($url, $element).autorefresh = true;
-                }
-            });
-
-            var $searchField = $('#menu input.search', $target);
-            // Remember initial search field value if any
-            if ($searchField.length && $searchField.val().length) {
-                _this.searchValue = $searchField.val();
-            }
         },
 
         /**
          * Global default event handlers
          */
         applyGlobalDefaults: function () {
-            // Apply element-specific behavior whenever the layout is rendered
-            // Note: It is important that this is the first handler for this event!
-            $(document).on('rendered', { self: this }, this.applyHandlers);
-
             $(document).on('visibilitychange', { self: this }, this.onVisibilityChange);
 
             $.each(this.icinga.behaviors, function (name, behavior) {
                 behavior.bind($(document));
             });
 
+            var _this = this;
+            $('.container').each(function () {
+                // Initialize module javascript (Applies only to module.js code)
+                var moduleName = $(this).data('icingaModule');
+                if (moduleName) {
+                    if (_this.icinga.hasModule(moduleName) && ! _this.icinga.isLoadedModule(moduleName)) {
+                        _this.icinga.loadModule(moduleName);
+                    }
+                }
+            });
+
             // We catch resize events
             $(window).on('resize', { self: this.icinga.ui }, this.icinga.ui.onWindowResize);
 
             // Trigger 'rendered' event also on page loads
-            $(window).on('load', { self: this }, this.onLoad);
+            $(document).on('icinga-init', { self: this }, this.onInit);
 
             // Destroy Icinga, clean up and interrupt pending requests on unload
             $( window ).on('unload', { self: this }, this.onUnload);
@@ -141,12 +75,15 @@
             $(document).on('focus', 'form select[data-related-radiobtn]', { self: this }, this.autoCheckRadioButton);
             $(document).on('focus', 'form input[data-related-radiobtn]', { self: this }, this.autoCheckRadioButton);
 
-            $(document).on('keyup', '#menu input.search', {self: this}, this.autoSubmitSearch);
+            $(document).on('rendered', '#menu', { self: this }, this.onRenderedMenu);
+            $(document).on('keyup', '#search', { self: this }, this.autoSubmitSearch);
 
             $(document).on('click', '.tree .handle', { self: this }, this.treeNodeToggle);
 
             $(document).on('click', '#search + .search-reset', this.clearSearch);
 
+            $(document).on('rendered', '.container', { self: this }, this.loadDashlets);
+
             // TBD: a global autocompletion handler
             // $(document).on('keyup', 'form.auto input', this.formChangeDelayed);
             // $(document).on('change', 'form.auto input', this.formChanged);
@@ -161,13 +98,17 @@
             } else {
                 $parent.addClass('collapsed');
             }
-
-            icinga.ui.fixControls($parent.closest('.container'));
         },
 
-        onLoad: function (event) {
+        onInit: function (event) {
             $('body').removeClass('loading');
-            //$('.container').trigger('rendered');
+
+            // Trigger the initial `rendered` events
+            $('.container').trigger('rendered');
+
+            // Additionally trigger a `rendered` event on the layout, some behaviors may
+            // want to differentiate whether a container or the entire layout is rendered
+            $('#layout').trigger('rendered');
         },
 
         onUnload: function (event) {
@@ -197,12 +138,25 @@
             return true;
         },
 
+        onRenderedMenu: function(event) {
+            var _this = event.data.self;
+            var $target = $(event.target);
+
+            var $searchField = $target.find('input.search');
+            // Remember initial search field value if any
+            if ($searchField.length && $searchField.val().length) {
+                _this.searchValue = $searchField.val();
+            }
+        },
+
         autoSubmitSearch: function(event) {
             var _this = event.data.self;
-            if ($('#menu input.search').val() === _this.searchValue) {
+            var $searchField = $(event.target);
+
+            if ($searchField.val() === _this.searchValue) {
                 return;
             }
-            _this.searchValue = $('#menu input.search').val();
+            _this.searchValue = $searchField.val();
 
             if (_this.searchTimer !== null) {
                 clearTimeout(_this.searchTimer);
@@ -210,11 +164,26 @@
             }
             var _event = $.extend({}, event);  // event seems gc'd once the timeout is over
             _this.searchTimer = setTimeout(function () {
-                _this.submitForm(_event, true);
+                _this.submitForm(_event, $searchField);
                 _this.searchTimer = null;
             }, 500);
         },
 
+        loadDashlets: function(event) {
+            var _this = event.data.self;
+            var $target = $(event.target);
+
+            if ($target.children('.dashboard').length) {
+                $target.find('.dashboard > .container').each(function () {
+                    var $dashlet = $(this);
+                    var url = $dashlet.data('icingaUrl');
+                    if (typeof url !== 'undefined') {
+                        _this.icinga.loader.loadUrl(url, $dashlet).autorefresh = true;
+                    }
+                });
+            }
+        },
+
         rememberSubmitButton: function(e) {
             var $button = $(this);
             var $form = $button.closest('form');
@@ -222,25 +191,19 @@
         },
 
         autoSubmitForm: function (event) {
-            return event.data.self.submitForm(event, true);
+            return event.data.self.submitForm(event, $(event.currentTarget));
         },
 
         /**
          *
          */
-        submitForm: function (event, autosubmit) {
+        submitForm: function (event, $autoSubmittedBy) {
             var _this   = event.data.self;
-            var icinga = _this.icinga;
+
             // .closest is not required unless subelements to trigger this
             var $form = $(event.currentTarget).closest('form');
-            var url = $form.attr('action');
-            var method = $form.attr('method');
-            var encoding = $form.attr('enctype');
-            var $button = $('input[type=submit]:focus', $form).add('button[type=submit]:focus', $form);
-            var progressTimer;
-            var $target;
-            var data;
 
+            var $button;
             var $rememberedSubmittButton = $form.data('submitButton');
             if (typeof $rememberedSubmittButton != 'undefined') {
                 if ($form.has($rememberedSubmittButton)) {
@@ -253,151 +216,32 @@
                 return true;
             }
 
-            if ($button.length === 0) {
+            if (typeof $button === 'undefined') {
                 var $el;
 
                 if (typeof event.originalEvent !== 'undefined'
                     && typeof event.originalEvent.explicitOriginalTarget === 'object') { // Firefox
                     $el = $(event.originalEvent.explicitOriginalTarget);
-                    icinga.logger.debug('events/submitForm: Button is event.originalEvent.explicitOriginalTarget');
+                    _this.icinga.logger.debug('events/submitForm: Button is event.originalEvent.explicitOriginalTarget');
                 } else {
                     $el = $(event.currentTarget);
-                    icinga.logger.debug('events/submitForm: Button is event.currentTarget');
+                    _this.icinga.logger.debug('events/submitForm: Button is event.currentTarget');
                 }
 
                 if ($el && ($el.is('input[type=submit]') || $el.is('button[type=submit]'))) {
                     $button = $el;
                 } else {
-                    icinga.logger.debug(
+                    _this.icinga.logger.debug(
                         'events/submitForm: Can not determine submit button, using the first one in form'
                     );
                 }
             }
 
-            if (typeof method === 'undefined') {
-                method = 'POST';
-            } else {
-                method = method.toUpperCase();
+            if (! $autoSubmittedBy && event.detail && event.detail.submittedBy) {
+                $autoSubmittedBy = $(event.detail.submittedBy);
             }
 
-            if (typeof encoding === 'undefined') {
-                encoding = 'application/x-www-form-urlencoded';
-            }
-
-            if (typeof autosubmit === 'undefined') {
-                autosubmit = false;
-            }
-
-            if ($button.length === 0) {
-                $button = $('input[type=submit]', $form).add('button[type=submit]', $form).first();
-            }
-
-            if ($button.length) {
-                // Activate spinner
-                if ($button.hasClass('spinner')) {
-                    $button.addClass('active');
-                }
-
-                $target = _this.getLinkTargetFor($button);
-            } else {
-                $target = _this.getLinkTargetFor($form);
-            }
-
-            if (! url) {
-                // Use the URL of the target container if the form's action is not set
-                url = $target.closest('.container').data('icinga-url');
-            }
-
-            icinga.logger.debug('Submitting form: ' + method + ' ' + url, method);
-
-            if (method === 'GET') {
-                var dataObj = $form.serializeObject();
-
-                if (! autosubmit) {
-                    if ($button.length && $button.attr('name') !== 'undefined') {
-                        dataObj[$button.attr('name')] = $button.attr('value');
-                    }
-                }
-
-                url = icinga.utils.addUrlParams(url, dataObj);
-            } else {
-                if (encoding === 'multipart/form-data') {
-                    data = new window.FormData($form[0]);
-                } else {
-                    data = $form.serializeArray();
-                }
-
-                if (! autosubmit) {
-                    if ($button.length && $button.attr('name') !== 'undefined') {
-                        if (encoding === 'multipart/form-data') {
-                            data.append($button.attr('name'), $button.attr('value'));
-                        } else {
-                            data.push({
-                                name: $button.attr('name'),
-                                value: $button.attr('value')
-                            });
-                        }
-                    }
-                }
-            }
-
-            // Disable all form controls to prevent resubmission except for our search input
-            // Note that disabled form inputs will not be enabled via JavaScript again
-            if ($target.attr('id') === $form.closest('.container').attr('id')) {
-                $form.find(':input:not(#search):not(:disabled)').prop('disabled', true);
-            }
-
-            // Show a spinner depending on how the form is being submitted
-            if (autosubmit && typeof $el !== 'undefined' && $el.siblings('.spinner').length) {
-                $el.siblings('.spinner').first().addClass('active');
-            } else if ($button.length && $button.is('button') && $button.hasClass('animated')) {
-                $button.addClass('active');
-            } else if ($button.length && $button.attr('data-progress-label')) {
-                var isInput = $button.is('input');
-                if (isInput) {
-                    $button.prop('value', $button.attr('data-progress-label') + '...');
-                } else {
-                    $button.html($button.attr('data-progress-label') + '...');
-                }
-
-                // Use a fixed width to prevent the button from wobbling
-                $button.css('width', $button.css('width'));
-
-                progressTimer = icinga.timer.register(function () {
-                    var label = isInput ? $button.prop('value') : $button.html();
-                    var dots = label.substr(-3);
-
-                    // Using empty spaces here to prevent centered labels from wobbling
-                    if (dots === '...') {
-                        label = label.slice(0, -2) + '  ';
-                    } else if (dots === '.. ') {
-                        label = label.slice(0, -1) + '.';
-                    } else if (dots === '.  ') {
-                        label = label.slice(0, -2) + '. ';
-                    }
-
-                    if (isInput) {
-                        $button.prop('value', label);
-                    } else {
-                        $button.html(label);
-                    }
-                }, null, 100);
-            } else if ($button.length && $button.next().hasClass('spinner')) {
-                $('i', $button.next()).addClass('active');
-            } else if ($form.attr('data-progress-element')) {
-                var $progressElement = $('#' + $form.attr('data-progress-element'));
-                if ($progressElement.length) {
-                    if ($progressElement.hasClass('spinner')) {
-                        $('i', $progressElement).addClass('active');
-                    } else {
-                        $('i.spinner', $progressElement).addClass('active');
-                    }
-                }
-            }
-
-            var req = icinga.loader.loadUrl(url, $target, data, method);
-            req.forceFocus = autosubmit ? $(event.currentTarget) : $button.length ? $button : null;
-            req.progressTimer = progressTimer;
+            _this.icinga.loader.submitForm($form, $autoSubmittedBy, $button);
 
             event.stopPropagation();
             event.preventDefault();
@@ -429,6 +273,11 @@
             var linkTarget = $a.attr('target');
             var $target;
             var formerUrl;
+
+            if (! href) {
+                return;
+            }
+
             if (href.match(/^(?:(?:mailto|javascript|data):|[a-z]+:\/\/)/)) {
                 event.stopPropagation();
                 return true;
@@ -516,7 +365,7 @@
                     }
                     return false;
                 }
-                $target = _this.getLinkTargetFor($a);
+                $target = icinga.loader.getLinkTargetFor($a);
 
                 formerUrl = $target.data('icingaUrl');
                 if (typeof formerUrl !== 'undefined' && formerUrl.split(/#/)[0] === href.split(/#/)[0]) {
@@ -528,7 +377,7 @@
                     return false;
                 }
             } else {
-                $target = _this.getLinkTargetFor($a);
+                $target = icinga.loader.getLinkTargetFor($a);
             }
 
             // Load link URL
@@ -542,63 +391,6 @@
             return false;
         },
 
-        /**
-         * Detect the link/form target for a given element (link, form, whatever)
-         */
-        getLinkTargetFor: function($el)
-        {
-            var targetId;
-
-            // If everything else fails, our target is the first column...
-            var $target = $('#col1');
-
-            // ...but usually we will use our own container...
-            var $container = $el.closest('.container');
-            if ($container.length) {
-                $target = $container;
-            }
-
-            // You can of course override the default behaviour:
-            if ($el.closest('[data-base-target]').length) {
-                targetId = $el.closest('[data-base-target]').data('baseTarget');
-
-                // Simulate _next to prepare migration to dynamic column layout
-                // YES, there are duplicate lines right now.
-                if (targetId === '_next') {
-                    if (this.icinga.ui.hasOnlyOneColumn()) {
-                        targetId = 'col1';
-                        $target = $('#' + targetId);
-                    } else {
-                        if ($el.closest('#col2').length) {
-                            this.icinga.ui.moveToLeft();
-                        }
-                        targetId = 'col2';
-                        $target = $('#' + targetId);
-                    }
-                } else if (targetId === '_self') {
-                    $target = $el.closest('.container');
-                    targetId = $target.attr('id');
-                } else if (targetId === '_main') {
-                    targetId = 'col1';
-                    $target = $('#' + targetId);
-                    this.icinga.ui.layout1col();
-                } else {
-                    $target = $('#' + targetId);
-                    if (! $target.length) {
-                        this.icinga.logger.warn('Link target "#' + targetId + '" does not exist in DOM.');
-                    }
-                }
-
-            }
-
-            // Hardcoded layout switch unless columns are dynamic
-            if ($target.attr('id') === 'col2') {
-                this.icinga.ui.layout2col();
-            }
-
-            return $target;
-        },
-
         clearSearch: function (event) {
             $(event.target).parent().find('#search').attr('value', '');
         },
