@@ -110,15 +110,12 @@
             $('link').each(function() {
                 var $oldLink = $(this);
                 if ($oldLink.hasAttr('type') && $oldLink.attr('type').indexOf('css') > -1) {
-                    var base = location.protocol + '//' + location.host;
-                    var url = icinga.utils.addUrlParams(
-                        $(this).attr('href'),
-                        { id: new Date().getTime() }
-                    );
-
                     var $newLink = $oldLink.clone().attr(
                         'href',
-                        base + '/' + url.replace(/^\//, '')
+                        icinga.utils.addUrlParams(
+                            $oldLink.attr('href'),
+                            { id: new Date().getTime() } // Only required for Firefox to reload CSS automatically
+                        )
                     ).on('load', function() {
                         $oldLink.remove();
                     });
@@ -340,6 +337,26 @@
             $('#layout').trigger('layout-change');
         },
 
+        prepareColumnFor: function ($el, $target) {
+            var explicitTarget;
+
+            if ($target.attr('id') === 'col2') {
+                if ($el.closest('#col2').length) {
+                    explicitTarget = $el.closest('[data-base-target]').data('baseTarget');
+                    if (typeof explicitTarget !== 'undefined' && explicitTarget === '_next') {
+                        this.moveToLeft();
+                    }
+                } else {
+                    this.layout2col();
+                }
+            } else { // if ($target.attr('id') === 'col1')
+                explicitTarget = $el.closest('[data-base-target]').data('baseTarget');
+                if (typeof explicitTarget !== 'undefined' && explicitTarget === '_main') {
+                    this.layout1col();
+                }
+            }
+        },
+
         getAvailableColumnSpace: function () {
             return $('#main').width() / this.getDefaultFontSize();
         },
@@ -536,24 +553,29 @@
             $('#layout').toggleClass('fullscreen-layout');
         },
 
-        getUniqueContainerId: function ($cont) {
-            if (typeof $cont === 'undefined' || !$cont.length) {
+        getUniqueContainerId: function (container) {
+            if (typeof container.jquery !== 'undefined') {
+                if (! container.length) {
+                    return null;
+                }
+
+                container = container[0];
+            } else if (typeof container === 'undefined') {
                 return null;
             }
 
-            var containerId = $cont.data('icingaContainerId');
-            if (typeof containerId === 'undefined') {
+            var containerId = container.dataset.icingaContainerId || null;
+            if (containerId === null) {
                 /**
                  * Only generate an id if it's not for col1 or the menu (which are using the non-suffixed window id).
                  * This is based on the assumption that the server only knows about the menu and first column
                  * and therefore does not need to protect its ids. (As the menu is most likely part of the sidebar)
                  */
-                if ($cont.attr('id') === 'menu' || $cont.attr('id') === 'col1' || $cont.closest('#col1').length) {
-                    return null;
+                var col1 = document.getElementById('col1');
+                if (container.id !== 'menu' && col1 !== null && ! col1.contains(container)) {
+                    containerId = this.icinga.utils.generateId(6); // Random because the content may move
+                    container.dataset.icingaContainerId = containerId;
                 }
-
-                containerId = this.icinga.utils.generateId(6); // Random because the content may move
-                $cont.data('icingaContainerId', containerId);
             }
 
             return containerId;
